
// 		*****************************************************************************
// 		*														  	AF LIBRARY					  								  		*
// 		*													Written By Ali Fallah												  		*
// 		*											95/11/21 = Thursday, February 9, 2017									*
// 		*****************************************************************************


#include ".\AF_CHLCD.h"
#include <lpc17xx.h>



/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
*/

/*--------------------- Clock Configuration ----------------------------------
//
// <e> CHAR LCD Setting
//
//		 <o1.0..2>   Connect Port LCD
//					<i>  you want Connect Char LCD to Which PORT?
//                     <0=> PORT 0
//                     <1=> PORT 1
//                     <2=> PORT 2
//                     <3=> PORT 3
//                     <4=> PORT 4
// 			<o2.0..4>   RS PIN LCD
//						<i>  You want Connect RS PIN LCD to Which pin? (0,1,...,31)
//						<0-31>
// 			<o3.0..4>   RW PIN LCD
//						<0-31>
// 			<o4.0..4>   E PIN LCD
//						<0-31>
// 			<o5.0..4>   DB4 PIN LCD
//						<0-31>
// 			<o6.0..4>   DB5 PIN LCD
//						<0-31>
// 			<o7.0..4>   DB6 PIN LCD
//						<0-31>
// 			<o8.0..4>   DB7 PIN LCD
//						<0-31>
//
// </e>
*/
#define CHLCD_ENABLE           1
#define CH_LCD_PORT   2
#define CH_LCD_RS     0
#define CH_LCD_RW     1
#define CH_LCD_E      2
#define CH_LCD_DB4    4
#define CH_LCD_DB5    5
#define CH_LCD_DB6    6
#define CH_LCD_DB7    7


#if (CHLCD_ENABLE==1)


#if    (CH_LCD_PORT==0)
#define   PIO_enable        PINSEL0
#define   PIO_SetOutput     LPC_GPIO0->FIOSET
#define   PIO_ClearOutput   LPC_GPIO0->FIOCLR
#define   PIO_CfgOutput     LPC_GPIO0->FIODIR
#endif

#if    (CH_LCD_PORT==1)
#define   PIO_enable        PINSEL2
#define   PIO_SetOutput     LPC_GPIO1->FIOSET   
#define   PIO_ClearOutput   LPC_GPIO1->FIOCLR  
#define   PIO_CfgOutput     LPC_GPIO1->FIODIR
#endif


#if    (CH_LCD_PORT==2)
#define   PIO_enable        PINSEL4
#define   PIO_SetOutput     LPC_GPIO2->FIOSET   
#define   PIO_ClearOutput   LPC_GPIO2->FIOCLR  
#define   PIO_CfgOutput     LPC_GPIO2->FIODIR
#endif


#if    (CH_LCD_PORT==3)
#define   PIO_enable       PINSEL6
#define   PIO_SetOutput    LPC_GPIO3->FIOSET   
#define   PIO_ClearOutput  LPC_GPIO3->FIOCLR  
#define   PIO_CfgOutput    LPC_GPIO3->FIODIR
#endif

#if    (CH_LCD_PORT==4)
#define   PIO_enable       PINSEL8
#define   PIO_SetOutput    LPC_GPIO4->FIOSET   
#define   PIO_ClearOutput  LPC_GPIO4->FIOCLR  
#define   PIO_CfgOutput    LPC_GPIO4->FIODIR
#endif

#endif // CHLCD_ENABLE

//============================================================================
void AF_CH_LCD::cwait (volatile int t) {
  for (;t; t--);
}
// ============================================================================
void AF_CH_LCD::setD4567(char val)
{
   PIO_ClearOutput =((1<<CH_LCD_DB4)|(1<<CH_LCD_DB5)|(1<<CH_LCD_DB6)|(1<<CH_LCD_DB7));
   if (val & 0x80)
      PIO_SetOutput =(1<<CH_LCD_DB7)   ;
   if (val & 0x40)
      PIO_SetOutput =(1<<CH_LCD_DB6);
   if (val & 0x20)
   PIO_SetOutput =(1<<CH_LCD_DB5);
   if (val & 0x10)
   PIO_SetOutput =(1<<CH_LCD_DB4);
}
// ============================================================================
void AF_CH_LCD:: PutChar(char ch)
{
   PIO_SetOutput =(1<<CH_LCD_RS);
   setD4567(ch);
   PIO_SetOutput=(1<<CH_LCD_E);
   clcd_minDelay();
   PIO_ClearOutput=(1<<CH_LCD_E);
   clcd_minDelay();
   setD4567(ch<<4);
   PIO_SetOutput=(1<<CH_LCD_E);
   clcd_minDelay();
PIO_ClearOutput=(1<<CH_LCD_E);
   clcd_Delay();
}
// ============================================================================
void AF_CH_LCD::Command(char cmd)   //Sends Command to LCD
{
   PIO_ClearOutput=(1<<CH_LCD_RS);
   setD4567(cmd);
   PIO_SetOutput =(1<<CH_LCD_E);
   clcd_minDelay();
   PIO_ClearOutput=(1<<CH_LCD_E);
   clcd_minDelay();
   setD4567(cmd<<4);
   PIO_SetOutput =(1<<CH_LCD_E);
   clcd_minDelay();
   PIO_ClearOutput =(1<<CH_LCD_E);
   clcd_Delay();
}
// ============================================================================
void AF_CH_LCD::Init()
{
   PIO_CfgOutput = ((1<<CH_LCD_RS)|(1<<CH_LCD_RW)|(1<<CH_LCD_E)|(1<<CH_LCD_DB4)|(1<<CH_LCD_DB5)|(1<<CH_LCD_DB6)|(1<<CH_LCD_DB7));
	 PIO_ClearOutput = (1<<CH_LCD_RW);
   clcd_Delay();
   setD4567(0);
   setD4567((1<<5)|(1<<4));
   PIO_SetOutput=(1<<CH_LCD_E);
   clcd_minDelay();
   PIO_ClearOutput =(1<<CH_LCD_E);
   clcd_Delay();
   setD4567((1<<5)|(1<<4));
   PIO_SetOutput =(1<<CH_LCD_E);
   clcd_minDelay();
   PIO_ClearOutput =(1<<CH_LCD_E);
   clcd_Delay();
   setD4567(1<<5);
   PIO_SetOutput =(1<<CH_LCD_E);
   clcd_minDelay();
   PIO_ClearOutput =(1<<CH_LCD_E);
   clcd_Delay();
   Command(0x28);
   Command(0x0c);
   clcd_Delay();
}
//============================================================================
void AF_CH_LCD::Gotoxy(char y, char x)   //Cursor to X Y position
{
   char DDRAMAddr;
x=x-1;
   switch(y)
   {
      case 1: DDRAMAddr = 0x00+x; break;
      case 2: DDRAMAddr = 0x40+x; break;
      case 3: DDRAMAddr = 0x14+x; break;
      case 4: DDRAMAddr = 0x54+x; break;
      default: DDRAMAddr = 0x00+x;
   }
   Command(1<<7 | DDRAMAddr);
   clcd_Delay();
}
// ============================================================================
void AF_CH_LCD::DefineChar(const char *pc,char char_code)
{
   char a , i;
   a = ((char_code<<3)|0x40) & 0xff;
   for (i = 0; i < 8 ;i++)
   {
      Command(a++);
      clcd_Delay();
      PutChar(pc[i]);
      clcd_Delay();
   }
}
//=============================================================================
void AF_CH_LCD::Clear(void)            //Clears LCD
{
   Command(0x01);
   clcd_Delay();
   Command(0x02);
   clcd_maxDelay();
}
//=============================================================================
void AF_CH_LCD::LeftToRight (char n)   //Scrol n of characters Right
{
   char i;
   for (i = 0 ; i < n ; i++)
   {
      Command(0x1E);
      clcd_Delay();
   }
}
//=============================================================================
void AF_CH_LCD::RightToLeft (char n)   //Scrol n of characters Left
{
   char q;
   for (q = 0 ;q < n ; q++)
   {
      Command(0x18);
      clcd_Delay();
   }
}
//============================================================================
 void AF_CH_LCD::Puts(int num)
{
   int i1,j1;
   int p,f=0;
   char ch[5];
 for(i1=0;i1<5;i1++)
   {
     p=1;
     for(j1=4-i1;j1>0;j1--)
       p = p*10;  
     ch[i1] = (num/p);
    if (num>=p && !f){
     f=1;
    }
     num =num - ch[i1] *p ;
    ch[i1] = ch[i1] +48;
     if(f) PutChar(ch[i1]);
   }
}
// ============================================================================
void AF_CH_LCD::Print(char* data)   //Outputs string to LCD
{
   char *p;
   p = data;
   if (!p)
      return;
   for(; *p ; )
   {
      PutChar(*p);
      p++;
   }
}
//=============================================================================
void AF_CH_LCD::DisplayOff(void)
{
   Command(0x08);
   clcd_Delay();
}
//=============================================================================
void AF_CH_LCD::DisplayOn(void)
{
   Command(0x0C);
   clcd_Delay();
}
//=============================================================================
void AF_CH_LCD::CursorOff(void)
{
   Command(0x0C);
   clcd_Delay();
}
//=============================================================================
void AF_CH_LCD::CursorOn(void)
{
   Command(0x0E);
   clcd_Delay();
}
//=============================================================================
void AF_CH_LCD::CursorBlinkOn(void)
{
   Command(0x0F);
   clcd_Delay();
}
//=============================================================================
void AF_CH_LCD::CursorBlinkOff(void)
{
   Command(0x0F);
   clcd_Delay();
}

//=============================================================================
void AF_CH_LCD::Home (void)
{
   Gotoxy(0,0);
}


